#include "ParseConfigFile.h"
#include <stdio.h>

HostNode *g_pHosts;
long g_nHosts;
//long g_nRootPort;
//long g_nFirstSMPProcs;
TCHAR g_pszExe[MAX_PATH], g_pszArgs[MAX_PATH], g_pszEnv[1024];
TCHAR g_pszJobID[100] = TEXT("MPICHJob");
//TCHAR g_pszFirstHost[100];
int g_nNproc = 1;
bool g_bStaticEnvironment = true;

// Function name	: ParseLineIntoHostNode
// Description	    : 
// Return type		: HostNode* 
// Argument         : LPTSTR line
HostNode* ParseLineIntoHostNode(LPTSTR line)
{
	TCHAR buffer[1024];
	LPTSTR pChar, pChar2;
	HostNode *node = NULL;

	_tcscpy(buffer, line);
	pChar = buffer;

	// Advance over white space
	while (*pChar != _T('\0') && _istspace(*pChar))
		pChar++;
	if (*pChar == _T('#') || *pChar == _T('\0'))
		return NULL;

	// Trim trailing white space
	pChar2 = &buffer[_tcslen(buffer)-1];
	while (_istspace(*pChar2) && (pChar >= pChar))
	{
		*pChar2 = '\0';
		pChar2--;
	}
	
	// If there is anything left on the line, consider it a host name
	if (_tcslen(pChar) > 0)
	{
		node = new HostNode;
		node->nSMPProcs = 1;
		node->next = NULL;
		node->exe[0] = _T('\0');

		// Copy the host name
		pChar2 = node->host;
		while (*pChar != _T('\0') && !_istspace(*pChar))
		{
			*pChar2 = *pChar;
			pChar++;
			pChar2++;
		}
		*pChar2 = _T('\0');

		// Advance over white space
		while (*pChar != _T('\0') && _istspace(*pChar))
			pChar++;
		// Get the number of SMP processes
		if (*pChar != _T('\0'))
		{
			node->nSMPProcs = _ttoi(pChar);
			if (node->nSMPProcs < 1)
				node->nSMPProcs = 1;
		}
		// Advance over the number
		while (*pChar != _T('\0') && _istdigit(*pChar))
			pChar++;

		// Advance over white space
		while (*pChar != _T('\0') && _istspace(*pChar))
			pChar++;
		// Copy the executable
		if (*pChar != _T('\0'))
			_tcscpy(node->exe, pChar);
	}

	return node;
}

// Function name	: ParseConfigFile
// Description	    : 
// Return type		: void 
// Argument         : LPTSTR filename
bool ParseConfigFile(LPTSTR filename)
{
	FILE *fin;
	TCHAR buffer[1024] = TEXT("");
	bool bIDParsed = false;
	bool bHostsParsed = false;

	fin = _tfopen(filename, TEXT("r"));
	if (fin == NULL)
	{
		_tprintf(TEXT("Unable to open file: %s\n"), filename);
		return false;
	}

	/*
	_fgetts(buffer, 1024, fin);
	if (_tcsnicmp(buffer, TEXT("MPICH Job 1.1.0"), 15) != 0)
	{
		_tprintf(TEXT("Valid config files must begin with 'MPICH Job 1.1.0'\n"));
		fclose(fin);
		ExitProcess(1);
	}
	//*/
	while (_fgetts(buffer, 1024, fin))
	{
		/*
		// Check for the name of the executable
		if (_tcsnicmp(buffer, TEXT("exe "), 4) == 0)
		{
			TCHAR *pChar = &buffer[4];
			while (_istspace(*pChar))
				pChar++;
			_tcscpy(g_pszExe, pChar);
			pChar = &g_pszExe[_tcslen(g_pszExe)-1];
			while (_istspace(*pChar) && (pChar >= g_pszExe))
			{
				*pChar = '\0';
				pChar--;
			}
		}
		else
		// Check for program arguments
		if (_tcsnicmp(buffer, TEXT("args "), 5) == 0)
		{
			TCHAR *pChar = &buffer[5];
			while (_istspace(*pChar))
				pChar++;
			_tcscpy(g_pszArgs, pChar);
			pChar = &g_pszArgs[_tcslen(g_pszArgs)-1];
			while (_istspace(*pChar) && (pChar >= g_pszArgs))
			{
				*pChar = '\0';
				pChar--;
			}
		}
		else
		//*/
		// Check for environment variables
		if (_tcsnicmp(buffer, TEXT("env "), 4) == 0)
		{
			TCHAR *pChar = &buffer[4];
			while (_istspace(*pChar))
				pChar++;
			_tcscpy(g_pszEnv, pChar);
			pChar = &g_pszEnv[_tcslen(g_pszEnv)-1];
			while (_istspace(*pChar) && (pChar >= g_pszEnv))
			{
				*pChar = '\0';
				pChar--;
			}
		}
		else
		// Check for the Job id
		if (_tcsnicmp(buffer, TEXT("id "), 3) == 0)
		{
			TCHAR *pChar = &buffer[3];
			while (_istspace(*pChar))
				pChar++;
			_tcscpy(g_pszJobID, pChar);
			pChar = &g_pszJobID[_tcslen(g_pszJobID)-1];
			while (_istspace(*pChar) && (pChar >= g_pszJobID))
			{
				*pChar = '\0';
				pChar--;
			}
			bIDParsed = true;
		}
		else
		// Check for the total number of processes
		if (_tcsnicmp(buffer, TEXT("nproc "), 6) == 0)
		{
			g_nNproc = _ttoi(&buffer[6]);
			g_bStaticEnvironment = true;
			bHostsParsed = true;
		}
		else
		// Check for hosts
		if (_tcsnicmp(buffer, TEXT("hosts"), 5) == 0)
		{
			g_bStaticEnvironment = false;
			g_nHosts = 0;
			g_pHosts = NULL;
			HostNode *node, dummy;
			dummy.next = NULL;
			node = &dummy;
			while (_fgetts(buffer, 1024, fin))
			{
				node->next = ParseLineIntoHostNode(buffer);
				if (node->next != NULL)
				{
					node = node->next;
					g_nHosts++;
				}
			}
			g_pHosts = dummy.next;

			fclose(fin);
			return bIDParsed;
		}
	}
	fclose(fin);
	return (bIDParsed && bHostsParsed);
}
